# Copyright 2021 Rafael Mardojai CM
# SPDX-License-Identifier: GPL-3.0-or-later

import os
from gettext import gettext as _
from random import randint

from gi.repository import Adw, Gio, GLib, Gtk

from blanket.define import RES_PATH
from blanket.settings import Settings


@Gtk.Template(resource_path=f"{RES_PATH}/preferences.ui")
class PreferencesDialog(Adw.PreferencesDialog):
    __gtype_name__ = "PreferencesDialog"

    dark_group: Adw.PreferencesGroup = Gtk.Template.Child()  # type: ignore
    dark: Adw.SwitchRow = Gtk.Template.Child()  # type: ignore
    autostart: Adw.SwitchRow = Gtk.Template.Child()  # type: ignore
    start_paused: Adw.SwitchRow = Gtk.Template.Child()  # type: ignore
    inhibition: Adw.SwitchRow = Gtk.Template.Child()  # type: ignore

    def __init__(self, window, **kwargs):
        super().__init__(**kwargs)

        self.window = window

        # Show dark theme preference if system does not support color schemes
        self.dark_group.props.visible = (
            not Adw.StyleManager.get_default().props.system_supports_color_schemes
        )
        # Dark theme
        Settings.get().bind(
            "dark-mode", self.dark, "active", Gio.SettingsBindFlags.DEFAULT
        )
        self.dark.connect("notify::active", self._toggle_dark)

        # Autostart
        self.autostart_failed = False
        self.autostart_saved = Settings.get().autostart
        self.autostart.props.active = self.autostart_saved
        self.autostart.connect("notify::active", self._toggle_autostart)

        # Start paused
        Settings.get().bind(
            "start-paused", self.start_paused, "active", Gio.SettingsBindFlags.DEFAULT
        )

        # Inhibition
        Settings.get().bind(
            "inhibit-suspension",
            self.inhibition,
            "active",
            Gio.SettingsBindFlags.DEFAULT,
        )

    def _toggle_dark(self, switch: Adw.SwitchRow, _pspec):
        style_manager = Adw.StyleManager.get_default()
        if switch.props.active:
            style_manager.props.color_scheme = Adw.ColorScheme.FORCE_DARK
        else:
            style_manager.props.color_scheme = Adw.ColorScheme.PREFER_LIGHT

    def _toggle_autostart(self, switch: Adw.SwitchRow, _pspec):
        self.__request_autostart(switch.props.active)

    def __request_autostart(self, active: bool):
        if self.autostart_failed:
            self.autostart_failed = False
            return

        bus = Gio.bus_get_sync(Gio.BusType.SESSION, None)
        proxy = Gio.DBusProxy.new_sync(
            bus,
            Gio.DBusProxyFlags.NONE,
            None,
            "org.freedesktop.portal.Desktop",
            "/org/freedesktop/portal/desktop",
            "org.freedesktop.portal.Background",
            None,
        )

        identifier = self.__get_window_identifier()
        token = 0 + randint(10000000, 90000000)
        options = {
            "handle_token": GLib.Variant("s", f"com/rafaelmardojai/Blanket/{token}"),
            "reason": GLib.Variant("s", _("Autostart Blanket in background.")),
            "autostart": GLib.Variant("b", active),
            "commandline": GLib.Variant("as", ["blanket", "--hidden"]),
            "dbus-activatable": GLib.Variant("b", False),
        }

        try:
            request = proxy.RequestBackground("(sa{sv})", identifier, options)  # type: ignore
            if request is None:
                raise Exception(
                    "The DBus proxy didn't return an object path."
                    + "\nThe portal can't subscribe to the signal."
                )

            bus.signal_subscribe(
                "org.freedesktop.portal.Desktop",
                "org.freedesktop.portal.Request",
                "Response",
                request,
                None,
                Gio.DBusSignalFlags.NO_MATCH_RULE,
                self.__receive_autostart,
                None,
            )

        except Exception as e:
            print(e)

            error_dialog = Adw.AlertDialog.new(
                _("Request error"), _("The autostart request failed.")
            )
            error_dialog.add_response("ok", _("Ok"))
            error_dialog.present(self.window)
            self.autostart_failed = True
            self.autostart.set_active(self.autostart_saved)

    def __receive_autostart(self, *args):
        self.window.present()

        active = self.autostart.get_active()
        state = args[5][0]
        autostart = args[5][1]["autostart"]

        if state == 0:
            pass
        elif state == 1:
            if active:
                error_dialog = Adw.AlertDialog.new(
                    _("Authorization failed"),
                    _(
                        "Make sure Blanket has permission to run in the background in Settings → Applications → Blanket and try again."
                    ),
                )
                error_dialog.add_response("ok", _("Ok"))
                error_dialog.present(self.window)
        elif state == 2:
            error_dialog = Adw.AlertDialog.new(
                _("Request error"), _("The autostart request failed.")
            )
            error_dialog.add_response("ok", _("Ok"))
            error_dialog.present(self.window)

        self.autostart.set_active(autostart)
        Settings.get().autostart = autostart
        return

    def __get_window_identifier(self):
        session = os.getenv("XDG_SESSION_TYPE")
        surface = self.window.get_surface()

        if session == "x11":
            return f"x11:{str(surface.get_xid())}"
        elif session == "wayland":
            return "wayland:"
        return ""



# Copyright 2025 Rafael Mardojai CM
# SPDX-License-Identifier: GPL-3.0-or-later

from gettext import gettext as _

from gi.repository import Adw, Gio, Gtk

from blanket.define import RES_PATH
from blanket.settings import Settings
from blanket.sound import Sound


@Gtk.Template(resource_path=f"{RES_PATH}/sound-rename-dialog.ui")
class SoundRenameDialog(Adw.Dialog):
    __gtype_name__ = "SoundRenameDialog"

    headerbar: Adw.HeaderBar = Gtk.Template.Child()  # type: ignore
    title_widget: Adw.WindowTitle = Gtk.Template.Child()  # type: ignore
    accept_btn: Gtk.Button = Gtk.Template.Child()  # type: ignore
    name_entry: Adw.EntryRow = Gtk.Template.Child()  # type: ignore

    def __init__(self, sound: Sound, index: int, **kwargs):
        super().__init__()

        self.sound = sound
        self.index = index
        app = Gio.Application.get_default()
        self.window = app.get_active_window()  # type: ignore

        self.set_title(_("Rename Sound"))
        self.title_widget.set_subtitle(self.sound.name)
        self.name_entry.set_text(self.sound.name)
        # Wire buttons
        self.accept_btn.connect("clicked", self._on_rename_sound)

        self.connect("realize", Gio.Application.get_default().unset_space_accel)  # type: ignore
        self.connect("closed", Gio.Application.get_default().set_space_accel)  # type: ignore

    @Gtk.Template.Callback()
    def _on_cancel_clicked(self, _button):
        self.close()

    @Gtk.Template.Callback()
    def _on_entry_changed(self, _entry):
        name = self.__get_name()

        if self.sound is not None and (
            self.sound.name == name or name in Settings.get().custom_audios
        ):
            self.accept_btn.set_sensitive(False)
            return

        if name:
            self.accept_btn.set_sensitive(True)
        else:
            self.accept_btn.set_sensitive(False)

    def _on_rename_sound(self, _button):
        new_name = self.__get_name()

        if new_name == self.sound.name:
            self.close()
            return

        if not new_name or new_name in Settings.get().custom_audios:
            self.__invalid_name()
            return

        Settings.get().rename_custom_audio(self.sound.name, new_name)
        self.sound.name = new_name
        self.sound.title = new_name

        self.close()

    def __get_name(self):
        name = self.name_entry.get_text()
        name = name.strip()  # Strip name

        return name

    def __invalid_name(self):
        pass



# Copyright 2020-2022 Rafael Mardojai CM
# SPDX-License-Identifier: GPL-3.0-or-later

from gi.repository import GObject, Gtk


class PlayPauseButton(Gtk.Button):
    __gtype_name__ = "PlayPauseButton"

    playing: bool = GObject.Property(type=bool, default=True)  # type: ignore

    def __init__(self):
        super().__init__()

        self.add_css_class("playpause")
        self.add_css_class("circular")
        self.add_css_class("suggested-action")

        self.connect("notify::playing", self._on_playing_changed)

    def _on_playing_changed(self, _object, _pspec):
        if self.playing:
            self.set_icon_name("media-playback-pause-symbolic")
        else:
            self.set_icon_name("media-playback-start-symbolic")



# Copyright 2021 Rafael Mardojai CM
# SPDX-License-Identifier: GPL-3.0-or-later

import uuid
from typing import Self

from gi.repository import Gio, GLib, GObject


class Settings(Gio.Settings):
    _instance: Self | None = None
    _presets_settings: dict[str, Gio.Settings] = {}

    __gsignals__ = {
        "preset-changed": (GObject.SIGNAL_RUN_FIRST, None, (str,)),
    }

    @classmethod
    def get(cls) -> Self:
        """Return an active instance of Settings."""
        if cls._instance is None:
            cls._instance = cls()
        return cls._instance

    def __init__(self):
        super().__init__(schema_id="com.rafaelmardojai.Blanket")
        self.migrate_legacy_volumes()

    """ Autostart """

    @property
    def autostart(self) -> bool:
        return self.get_boolean("autostart")

    @autostart.setter
    def autostart(self, autostart: bool):
        self.set_boolean("autostart", autostart)

    """ Start in Pause """

    @property
    def start_paused(self) -> bool:
        return self.get_boolean("start-paused")

    @start_paused.setter
    def start_paused(self, paused: bool):
        self.set_boolean("start-paused", paused)

    """ Dark Mode """

    @property
    def dark_mode(self) -> bool:
        return self.get_boolean("dark-mode")

    @dark_mode.setter
    def dark_mode(self, dark: bool):
        self.set_boolean("dark-mode", dark)

    """ Inhibit suspension """

    @property
    def inhibit_suspension(self) -> bool:
        return self.get_boolean("inhibit-suspension")

    @inhibit_suspension.setter
    def inhibit_suspension(self, val: bool):
        self.set_boolean("inhibit-suspension", val)

    """ Sounds view scroll position """

    @property
    def scroll_position(self) -> float:
        return self.get_double("scroll-position")

    @scroll_position.setter
    def scroll_position(self, position: float):
        self.set_double("scroll-position", position)

    """ General volume level """

    @property
    def volume(self) -> float:
        return self.get_double("volume")

    @volume.setter
    def volume(self, volume: float):
        self.set_double("volume", volume)

    """ Playing state """

    @property
    def playing(self) -> bool:
        return self.get_boolean("playing")

    @playing.setter
    def playing(self, state: bool):
        self.set_boolean("playing", state)

    """ Background playing """

    @property
    def background(self) -> bool:
        return self.get_boolean("background-playback")

    @background.setter
    def background(self, background: bool):
        self.set_boolean("background-playback", background)

    """ Custom sounds """

    @property
    def custom_audios(self) -> dict[str, str]:
        return dict(self.get_value("custom-audios"))

    @custom_audios.setter
    def custom_audios(self, audios: dict[str, str]):
        self.set_value("custom-audios", GLib.Variant("a{ss}", audios))

    """ Custom sounds helper functions  """

    def add_custom_audio(self, name: str, uri: str):
        # Add the audio if the name is not already present
        if name not in self.custom_audios:
            saved_audios = self.custom_audios
            saved_audios[name] = uri
            self.custom_audios = saved_audios
        else:
            # TODO: Do something if True
            pass

    def remove_custom_audio(self, name: str):
        if name in self.custom_audios:
            saved_audios = self.custom_audios
            del saved_audios[name]
            self.custom_audios = saved_audios

            """
            # Also remove sound from volume dict
            if name in self.volume:
                del self.volume[name]
            """

    def rename_custom_audio(self, old_name: str, new_name: str):
        if old_name not in self.custom_audios:
            return

        if new_name in self.custom_audios:
            # TODO: Do something if True
            return

        saved_audios = self.custom_audios
        uri = saved_audios[old_name]
        del saved_audios[old_name]
        saved_audios[new_name] = uri
        self.custom_audios = saved_audios

    """ Presets """

    @property
    def presets(self) -> list[str]:
        return list(self.get_strv("presets"))

    @presets.setter
    def presets(self, presets: list[str]):
        self.set_strv("presets", presets)

    @property
    def active_preset(self) -> str:
        return self.get_string("active-preset")

    @active_preset.setter
    def active_preset(self, preset: str):
        self.set_string("active-preset", preset)
        self.emit("preset-changed", preset)

    @property
    def active_preset_name(self) -> str:
        return self.get_preset_name(self.active_preset)

    @property
    def default_preset(self) -> str:
        """Return the default preset ID."""
        return self.get_default_value("active-preset").get_string()  # type: ignore

    """ Presets helper functions  """

    def get_presets_dict(self) -> dict[str, str]:
        """Return the presets formatted as dict: ID => name."""
        presets_dict = {}
        for preset in self.presets:
            presets_dict[preset] = self.get_preset_name(preset)

        return presets_dict

    def add_preset(self, name: str) -> str:
        """Create a new Preset."""
        preset_id = str(uuid.uuid4())

        saved_presets = self.presets
        saved_presets.append(preset_id)
        self.presets = saved_presets

        # Set new preset name
        self.set_preset_name(preset_id, name)
        # Clone current preset to new
        current_volumes = self.get_preset_volumes(self.active_preset)
        current_mutes = self.get_preset_mutes(self.active_preset)
        self.set_preset_volumes(preset_id, current_volumes)
        self.set_preset_mutes(preset_id, current_mutes)

        return preset_id

    def remove_preset(self, preset_id: str) -> None | int:
        index = None

        if preset_id in self.presets:
            saved_presets = self.presets
            index = saved_presets.index(preset_id)
            saved_presets.remove(preset_id)
            self.presets = saved_presets

            # Remove settings instance
            if preset_id in self._presets_settings:
                del self._presets_settings[preset_id]

        # Return the index where the preset where positioned
        return index

    def get_preset_name(self, preset_id: str) -> str:
        settings = self.get_preset_settings(preset_id)
        return settings.get_string("visible-name")

    def set_preset_name(self, preset_id: str, name: str):
        preset = self.get_preset_settings(preset_id)
        preset.set_string("visible-name", name)
        preset.apply()  # Always apply changes on name change

    def get_preset_volumes(self, preset_id: str) -> dict[str, float]:
        settings = self.get_preset_settings(preset_id)
        return dict(settings.get_value("sounds-volume"))

    def set_preset_volumes(self, preset_id: str, volumes: dict[str, float]):
        settings = self.get_preset_settings(preset_id)
        settings.set_value("sounds-volume", GLib.Variant("a{sd}", volumes))

    def get_preset_mutes(self, preset_id: str) -> dict[str, bool]:
        settings = self.get_preset_settings(preset_id)
        return dict(settings.get_value("sounds-mute"))

    def set_preset_mutes(self, preset_id: str, mutes: dict[str, bool]):
        settings = self.get_preset_settings(preset_id)
        settings.set_value("sounds-mute", GLib.Variant("a{sb}", mutes))

    def get_preset_hide_inactive(self, preset_id: str) -> bool:
        settings = self.get_preset_settings(preset_id)
        return settings.get_boolean("hide-inactive")

    def set_preset_hide_inactive(self, preset_id: str, hide: bool):
        settings = self.get_preset_settings(preset_id)
        settings.set_boolean("hide-inactive", hide)

    """ Preset sound volume """

    def get_sound_volume(self, name: str):
        volumes = self.get_preset_volumes(self.active_preset)
        # If sound is set on volume dict
        if name in volumes:
            volume = volumes[name]
            if volume:
                return volume
        return 0.0

    def set_sound_volume(self, name: str, volume: float):
        saved_volumes = self.get_preset_volumes(self.active_preset)
        saved_volumes[name] = volume
        self.set_preset_volumes(self.active_preset, saved_volumes)

        if volume == 0:
            self.set_sound_mute(name, False)

    """ Preset sound mute """

    def get_sound_mute(self, name: str) -> bool:
        mutes = self.get_preset_mutes(self.active_preset)
        # If sound is set on mute dict
        if name in mutes:
            mute = mutes[name]
            return mute
        return True

    def set_sound_mute(self, name: str, mute: bool):
        saved_mutes = self.get_preset_mutes(self.active_preset)
        saved_mutes[name] = mute
        self.set_preset_mutes(self.active_preset, saved_mutes)

    """ Preset settings helper functions """

    def get_preset_settings(self, preset_id: str | None = None) -> Gio.Settings:
        """Get Preset Gio.Settings instance."""

        # By default return active preset
        if preset_id is None:
            preset_id = self.active_preset

        if preset_id not in self._presets_settings:
            path = self.get_property("path")
            if not path.endswith("/"):
                path += "/"
            path += preset_id + "/"
            self._presets_settings[preset_id] = Gio.Settings.new_with_path(
                "com.rafaelmardojai.Blanket.preset", path
            )
            # Set on ‘delay-apply’ mode so it only applies changes when we want
            self._presets_settings[preset_id].delay()

        return self._presets_settings[preset_id]

    def save_presets(self, preset_id: str | None = None):
        """Apply all the changes made to presets,
        or to a specific preset."""

        if preset_id is not None and preset_id in self._presets_settings:
            self._presets_settings[preset_id].apply()
        else:
            for key, settings in self._presets_settings.items():
                settings.apply()

    """ Legacy sounds volume """

    @property
    def legacy_sounds_volume(self) -> dict[str, float]:
        return dict(self.get_value("sounds-volume"))

    @legacy_sounds_volume.setter
    def legacy_sounds_volume(self, volumes_dict: dict[str, float]):
        self.set_value("sounds-volume", GLib.Variant("a{sd}", volumes_dict))

    def migrate_legacy_volumes(self):
        """Migrate legacy volumes to Default preset."""
        legacy_volumes = self.legacy_sounds_volume
        if legacy_volumes:
            self.set_preset_volumes(self.default_preset, legacy_volumes)
            self.legacy_sounds_volume = {}



# Copyright 2020-2021 Rafael Mardojai CM
# SPDX-License-Identifier: GPL-3.0-or-later

import sys
from gettext import gettext as _

import gi

try:
    gi.require_version("Adw", "1")
    gi.require_version("Gdk", "4.0")
    gi.require_version("Gst", "1.0")
    gi.require_version("GstPlay", "1.0")
    gi.require_version("Gtk", "4.0")
    from gi.repository import Adw, Gio, GLib, Gst, Gtk

    # Init GStreamer
    Gst.init(None)
except ImportError or ValueError as exc:
    print("Error: Dependencies not met.", exc)
    exit()

from blanket.define import ARTISTS, AUTHORS, RES_PATH, SOUND_ARTISTS, SOUND_EDITORS
from blanket.main_player import MainPlayer
from blanket.mpris import MPRIS
from blanket.preferences import PreferencesDialog
from blanket.settings import Settings
from blanket.widgets import PresetDialog
from blanket.widgets.sound_rename_dialog import SoundRenameDialog
from blanket.window import BlanketWindow


class Application(Adw.Application):
    def __init__(self, version):
        super().__init__(
            application_id="com.rafaelmardojai.Blanket",
            flags=Gio.ApplicationFlags.HANDLES_COMMAND_LINE,
        )
        GLib.set_application_name(_("Blanket"))
        GLib.setenv(
            "PULSE_PROP_application.icon_name",
            "com.rafaelmardojai.Blanket-symbolic",
            True,
        )
        # Connect app shutdown signal
        self.connect("shutdown", self._on_shutdown)

        # Track power status
        self.power_monitor = Gio.PowerProfileMonitor.dup_default()
        self.power_monitor.connect(
            "notify::power-saver-enabled", self._on_notify_power_saver_enabled
        )

        # Add --hidden command line option
        self.add_main_option(
            "hidden",
            ord("h"),
            GLib.OptionFlags.NONE,
            GLib.OptionArg.NONE,
            "Start window hidden",
            None,
        )
        # App window
        self.window: BlanketWindow | None = None
        self.window_hidden = False
        # App version
        self.version = version

    def do_startup(self):
        # Startup application
        Adw.Application.do_startup(self)

        style_manager = Adw.StyleManager.get_default()
        # if the system doesn't support libadwaita color schemes,
        # fall back to our setting
        if (
            Settings.get().dark_mode
            and not style_manager.props.system_supports_color_schemes
        ):
            style_manager.props.color_scheme = Adw.ColorScheme.FORCE_DARK

        # Start MPRIS server
        self.mpris = MPRIS(self)

        self.setup_actions()

    def setup_actions(self):
        # Quit application
        action = Gio.SimpleAction.new("quit", None)
        action.connect("activate", self.on_quit)
        self.add_action(action)

        # Show about dialog
        action = Gio.SimpleAction.new("about", None)
        action.connect("activate", self.on_about)
        self.add_action(action)

        # Show preferences dialog
        action = Gio.SimpleAction.new("preferences", None)
        action.connect("activate", self.on_preferences)
        self.add_action(action)

        # Toggle background-playback setting
        action = Gio.SimpleAction.new_stateful(
            "background-playback", None, Settings.get().get_value("background-playback")
        )
        action.connect("change-state", self.on_background)
        self.add_action(action)

        # Toggle playback
        action = Gio.SimpleAction.new("playpause", None)
        action.connect("activate", self.on_playpause)
        self.add_action(action)

        action = Gio.SimpleAction.new("play", None)
        action.connect("activate", self.on_play)
        self.add_action(action)

        # Create new preset from active
        action = Gio.SimpleAction.new("add-preset", None)
        action.connect("activate", self.on_add_preset)
        self.add_action(action)

        # Reset active preset volumes
        action = Gio.SimpleAction.new("reset-volumes", None)
        action.connect("activate", self.on_reset_volumes)
        self.add_action(action)

        # Add sound file
        action = Gio.SimpleAction.new("open", None)
        action.connect("activate", self.on_open)
        self.add_action(action)

        # Remove sound file
        action = Gio.SimpleAction.new("remove-sound", GLib.VariantType("u"))
        action.connect("activate", self.on_remove_sound)
        self.add_action(action)

        # Rename sound file
        action = Gio.SimpleAction.new("rename-sound", GLib.VariantType("u"))
        action.connect("activate", self.on_rename_sound)
        self.add_action(action)

        # Setup accelerator
        self.set_accels_for_action("app.quit", ["<Ctl>q"])
        self.set_accels_for_action("app.preferences", ["<Ctl>comma"])
        self.set_accels_for_action("app.playpause", ["<Ctl>m", "space"])
        self.set_accels_for_action("app.open", ["<Ctl>o"])
        self.set_accels_for_action("win.close", ["<Ctl>w"])
        self.set_accels_for_action("win.hide-inactive", ["<Ctl>h"])

    def do_activate(self):
        self.window = self.props.active_window  # type: ignore
        if not self.window:
            self.window = BlanketWindow(application=self)

        self.window.props.hide_on_close = Settings.get().background

        if self.window_hidden:
            self.window.hide()
            self.window_hidden = False
        else:
            self.window.present()

        # Connect window close-request signal to _on_window_close_request
        self.window.connect("close-request", self._on_window_close_request)

        # Load saved props
        MainPlayer.get().volume = Settings.get().volume
        MainPlayer.get().playing = (
            Settings.get().playing and not Settings.get().start_paused
        )

    def do_command_line(self, command_line):
        options = command_line.get_options_dict()
        options = options.end().unpack()

        if "hidden" in options and self.window is None:
            self.window_hidden = True

        self.activate()
        return 0

    def on_open(self, _action, _param):
        if self.window:
            self.window.open_audio()  # type: ignore

    def on_playpause(self, _action=None, _param=None):
        MainPlayer.get().playing = not MainPlayer.get().playing
        if self.window:
            self.window.hide_power_toast()

    def on_play(self, _action=None, _param=None):
        MainPlayer.get().playing = True
        if self.window:
            self.window.hide_power_toast()

    def on_pause(self, _action=None, _param=None):
        MainPlayer.get().playing = False
        if self.window:
            self.window.hide_power_toast()

    def on_next(self, _action=None, _param=None):
        MainPlayer.get().next_preset()

    def on_prev(self, _action=None, _param=None):
        MainPlayer.get().prev_preset()

    def on_reset_volumes(self, _action, _param):
        MainPlayer.get().reset_volumes()

    def on_add_preset(self, _action, _param):
        dialog = PresetDialog()
        dialog.present(self.window)

    def on_remove_sound(self, _action, index_variant: GLib.Variant):
        index = index_variant.get_uint32()
        sound = MainPlayer.get().get_by_index(index)

        if sound and index:
            sound.remove()  # type: ignore
            MainPlayer.get().remove(index)

    def on_rename_sound(self, _action, index_variant: GLib.Variant):
        # Open edit dialog
        index = index_variant.get_uint32()
        sound = MainPlayer.get().get_by_index(index)
        if sound and index:
            dialog = SoundRenameDialog(sound, index)  # type: ignore
            dialog.present(self.window)

    def on_background(self, action, value):
        action.set_state(value)
        Settings.get().background = value
        if self.window:
            self.window.props.hide_on_close = value

    def on_preferences(self, _action, _param):
        prefs = PreferencesDialog(self.window)
        prefs.present(self.window)

    def on_about(self, _action, _param):
        builder = Gtk.Builder.new_from_resource(f"{RES_PATH}/about.ui")
        about: Adw.AboutDialog = builder.get_object("about")  # type: ignore

        artists = self.__get_credits_list(ARTISTS)
        sound_artists = self.__get_credits_list(SOUND_ARTISTS)
        sound_editors = self.__get_credits_list(SOUND_EDITORS)

        about.set_version(self.version)
        about.set_developers(AUTHORS)
        about.set_designers(artists)
        about.add_link(_("Source Code"), "https://github.com/rafaelmardojai/blanket")
        about.add_credit_section(_("Sounds by"), sound_artists)
        about.add_credit_section(_("Sounds edited by"), sound_editors)

        about.present(self.window)

    def set_space_accel(self, _action):
        self.set_accels_for_action("app.playpause", ["<Ctl>m", "space"])

    def unset_space_accel(self, _action):
        self.set_accels_for_action("app.playpause", ["<Ctl>m"])

    def on_quit(self, _action, _param):
        self.quit()

    def _on_notify_power_saver_enabled(self, obj, pspec):
        if self.power_monitor.get_power_saver_enabled() and MainPlayer.get().playing:
            # Pause playback if the system enters power saver mode, as audio
            # playback uses quite a lot of power. Don’t re-enable it when
            # exiting power saver mode, as that would be jarring for the user.
            MainPlayer.get().playing = False
            if self.window:
                self.window.show_power_toast()

    def _save_settings(self):
        # Save mainplayer volume
        Settings.get().volume = MainPlayer.get().volume
        # Save mainplayer playing state
        Settings.get().playing = MainPlayer.get().playing
        # Save presets settings
        Settings.get().save_presets()

    def _on_window_close_request(self, window):
        if Settings.get().background:
            self._save_settings()  # Save settings
            window.hide()
        else:
            self.quit_from_window = True
            self.quit()

    def _on_shutdown(self, _app):
        self._save_settings()

    def __get_credits_list(self, dict_):
        credits_list = []
        for k, vs in dict_.items():
            s = k + ": " + ", ".join(vs)
            credits_list.append(s)
        return credits_list


def main(version):
    app = Application(version)
    return app.run(sys.argv)



# Copyright 2020-2022 Rafael Mardojai CM
# SPDX-License-Identifier: GPL-3.0-or-later

from gi.repository import Gst, GstPlay

from blanket.main_player import MainPlayer


class Player(GstPlay.Play):
    """
    GstPlay.Play with modifications
    """

    __gtype_name__ = "SoundPlayer"

    def __init__(self, sound):
        super().__init__()

        self.sound = sound
        # Create a var to save saved volume
        self.saved_volume = 0.0
        # Always start with volume in 0
        self.set_volume(0)

        # Set Sound.uri as player uri
        self.set_uri(self.sound.uri)
        self.name = self.sound.name

        # Loop setup
        self.prerolled = False
        self.pipeline = self.get_pipeline()
        bus = self.pipeline.get_bus()
        bus.add_signal_watch()
        bus.connect("message", self._on_bus_message)

        # Connect mainplayer volume signal
        self.volume_hdlr = MainPlayer.get().connect(
            "notify::volume", self._on_main_volume_changed
        )
        # Connect mainplayer muted signal
        self.playing_hdlr = MainPlayer.get().connect(
            "notify::playing", self._on_playing_changed
        )

        # Connect volume-changed signal
        self.connect("notify::volume", self._on_volume_changed)

    def set_virtual_volume(self, volume: float):
        # Get last saved sound volume
        self.saved_volume = volume
        # Multiply sound volume with mainplayer volume
        volume = self.saved_volume * MainPlayer.get().volume
        # Set final volume to player
        self.set_volume(volume)

    def remove(self):
        # Stop player
        self.stop()
        # Disconnect main player signals
        MainPlayer.get().disconnect(self.volume_hdlr)
        MainPlayer.get().disconnect(self.playing_hdlr)

    def _on_playing_changed(self, _player, _volume):
        if not self.__vol_zero():
            if MainPlayer.get().playing:
                self.play()
            else:
                self.pause()

    def _on_volume_changed(self, _player, _volume):
        # Fix external changes to player volume
        volume = self.saved_volume * MainPlayer.get().volume
        if volume > 0 and self.get_volume() == 0.0:
            self.set_volume(volume)
            return
        # Only play if volume > 0
        if self.__vol_zero():
            self.pause()
        elif MainPlayer.get().playing:
            self.play()

    def _on_main_volume_changed(self, _player, _volume):
        if not self.__vol_zero(self.sound.saved_volume):
            # Set volume again when mainplayer volume changes
            self.set_virtual_volume(self.saved_volume)

    def _on_bus_message(self, _bus, message: Gst.Message):
        if message:
            if message.type is Gst.MessageType.SEGMENT_DONE:
                self.pipeline.seek_simple(Gst.Format.TIME, Gst.SeekFlags.SEGMENT, 0)

            if message.type is Gst.MessageType.ASYNC_DONE:
                if not self.prerolled:
                    self.pipeline.seek_simple(
                        Gst.Format.TIME, Gst.SeekFlags.FLUSH | Gst.SeekFlags.SEGMENT, 0
                    )
                    self.prerolled = True

            return True

    def __vol_zero(self, volume: float | None = None):
        volume = volume if volume else self.get_volume()
        return True if volume == 0 else False



# Copyright 2021-2022 Rafael Mardojai CM
# SPDX-License-Identifier: GPL-3.0-or-later

from gi.repository import Gio, GObject

from blanket.settings import Settings


class Preset(GObject.Object):
    __gtype_name__ = "Preset"

    name: str = GObject.Property(type=str)  # type: ignore
    active: bool = GObject.Property(type=bool, default=False)  # type: ignore

    def __init__(self, preset_id: str):
        super().__init__()

        self.id = preset_id

        # Active state
        self.active = Settings.get().active_preset == self.id
        Settings.get().connect("changed::active-preset", self._on_active_preset_changed)

        # Bind preset name with settings one
        Settings.get().get_preset_settings(self.id).bind(
            "visible-name", self, "name", Gio.SettingsBindFlags.DEFAULT
        )

    def remove(self) -> int | None:
        if self.id != Settings.get().default_preset:
            index = Settings.get().remove_preset(self.id)
            return index  # Return the index where the preset where positioned
        return None

    def _on_active_preset_changed(self, _settings, _key):
        self.active = Settings.get().active_preset == self.id



# Forked from:
# https://gitlab.gnome.org/World/lollypop/-/blob/master/lollypop/mpris.py
#
# Copyright (c) 2014-2020 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
# Copyright (c) 2016 Gaurav Narula
# Copyright (c) 2016 Felipe Borges <felipeborges@gnome.org>
# Copyright (c) 2013 Arnel A. Borja <kyoushuu@yahoo.com>
# Copyright (c) 2013 Vadim Rutkovsky <vrutkovs@redhat.com>
# Copyright (c) 2020 Rafael Mardojai CM
# SPDX-License-Identifier: GPL-3.0-or-later

from gettext import gettext as _
from random import randint

from gi.repository import Gdk, Gio, GLib

from blanket.define import RES_PATH
from blanket.main_player import MainPlayer
from blanket.settings import Settings


class Server:
    def __init__(self, con, path):
        method_outargs = {}
        method_inargs = {}
        for interface in Gio.DBusNodeInfo.new_for_xml(self.__doc__ or "").interfaces:
            for method in interface.methods:
                method_outargs[method.name] = (
                    "(" + "".join([arg.signature for arg in method.out_args]) + ")"
                )
                method_inargs[method.name] = tuple(
                    arg.signature for arg in method.in_args
                )

            con.register_object(
                object_path=path,
                interface_info=interface,
                method_call_closure=self.on_method_call,
            )

        self.method_inargs = method_inargs
        self.method_outargs = method_outargs

    def on_method_call(
        self,
        connection,
        sender,
        object_path,
        interface_name,
        method_name,
        parameters,
        invocation,
    ):
        args = list(parameters.unpack())
        for i, sig in enumerate(self.method_inargs[method_name]):
            if sig == "h":
                msg = invocation.get_message()
                fd_list = msg.get_unix_fd_list()
                args[i] = fd_list.get(args[i])

        try:
            result = getattr(self, method_name)(*args)

            # out_args is at least (signature1).
            # We therefore always wrap the result as a tuple.
            # Refer to https://bugzilla.gnome.org/show_bug.cgi?id=765603
            result = (result,)

            out_args = self.method_outargs[method_name]
            if out_args != "()":
                variant = GLib.Variant(out_args, result)
                invocation.return_value(variant)
            else:
                invocation.return_value(None)
        except Exception as e:
            print(e)


class MPRIS(Server):
    """
    <!DOCTYPE node PUBLIC
    "-//freedesktop//DTD D-BUS Object Introspection 1.0//EN"
    "http://www.freedesktop.org/standards/dbus/1.0/introspect.dtd">
    <node>
        <interface name="org.freedesktop.DBus.Introspectable">
            <method name="Introspect">
                <arg name="data" direction="out" type="s"/>
            </method>
        </interface>
        <interface name="org.freedesktop.DBus.Properties">
            <method name="Get">
                <arg name="interface" direction="in" type="s"/>
                <arg name="property" direction="in" type="s"/>
                <arg name="value" direction="out" type="v"/>
            </method>
            <method name="Set">
                <arg name="interface_name" direction="in" type="s"/>
                <arg name="property_name" direction="in" type="s"/>
                <arg name="value" direction="in" type="v"/>
            </method>
            <method name="GetAll">
                <arg name="interface" direction="in" type="s"/>
                <arg name="properties" direction="out" type="a{sv}"/>
            </method>
        </interface>
        <interface name="org.mpris.MediaPlayer2">
            <method name="Raise">
            </method>
            <method name="Quit">
            </method>
            <property name="CanQuit" type="b" access="read" />
            <property name="CanRaise" type="b" access="read" />
            <property name="Identity" type="s" access="read"/>
            <property name="DesktopEntry" type="s" access="read"/>
        </interface>
        <interface name="org.mpris.MediaPlayer2.Player">
            <method name="Next"/>
            <method name="Previous"/>
            <method name="PlayPause"/>
            <method name="Play"/>
            <method name="Pause"/>
            <method name="Stop"/>
            <property name="PlaybackStatus" type="s" access="read"/>
            <property name="Metadata" type="a{sv}" access="read">
            </property>
            <property name="Volume" type="d" access="readwrite"/>
            <property name="CanGoNext" type="b" access="read"/>
            <property name="CanGoPrevious" type="b" access="read"/>
            <property name="CanPlay" type="b" access="read"/>
            <property name="CanPause" type="b" access="read"/>
            <property name="CanControl" type="b" access="read"/>
        </interface>
    </node>
    """

    __MPRIS_IFACE = "org.mpris.MediaPlayer2"
    __MPRIS_PLAYER_IFACE = "org.mpris.MediaPlayer2.Player"
    __MPRIS_BLANKET = "org.mpris.MediaPlayer2.Blanket"
    __MPRIS_PATH = "/org/mpris/MediaPlayer2"

    def __init__(self, app):
        self.app = app

        self.__metadata = {}

        track_id = 0 + randint(10000000, 90000000)
        self.__metadata["mpris:trackid"] = GLib.Variant(
            "o", f"{RES_PATH}/Track/{track_id}"
        )
        self.__metadata["xesam:title"] = GLib.Variant(
            "s", _(Settings.get().active_preset_name)
        )
        self.__metadata["xesam:album"] = GLib.Variant(
            "s", _("Listen to different sounds")
        )
        self.__metadata["xesam:artist"] = GLib.Variant("as", [_("Blanket")])

        self.__bus = Gio.bus_get_sync(Gio.BusType.SESSION, None)
        Gio.bus_own_name_on_connection(
            self.__bus, self.__MPRIS_BLANKET, Gio.BusNameOwnerFlags.NONE, None, None
        )
        Server.__init__(self, self.__bus, self.__MPRIS_PATH)

        MainPlayer.get().connect("preset-changed", self._on_preset_changed)
        MainPlayer.get().connect("notify::playing", self._on_playing_changed)
        MainPlayer.get().connect("notify::volume", self._on_volume_changed)

    def Raise(self):
        self.app.window.present_with_time(Gdk.CURRENT_TIME)

    def Quit(self):
        self.app.quit()

    def Next(self):
        self.app.on_next()

    def Previous(self):
        self.app.on_prev()

    def PlayPause(self):
        self.app.on_playpause()

    def Play(self):
        self.app.on_play()

    def Pause(self):
        self.app.on_pause()

    def Stop(self):
        self.app.on_pause()

    def Get(self, interface, property_name):
        if property_name in [
            "CanQuit",
            "CanRaise",
            "CanControl",
            "CanPlay",
            "CanPause",
        ]:
            return GLib.Variant("b", True)
        elif property_name == "CanGoNext":
            return GLib.Variant("b", MainPlayer.get().can_next)
        elif property_name == "CanGoPrevious":
            return GLib.Variant("b", MainPlayer.get().can_prev)
        elif property_name == "Identity":
            return GLib.Variant("s", "Blanket")
        elif property_name == "DesktopEntry":
            return GLib.Variant("s", "com.rafaelmardojai.Blanket")
        elif property_name == "PlaybackStatus":
            return GLib.Variant("s", self._get_status())
        elif property_name == "Metadata":
            return GLib.Variant("a{sv}", self.__metadata)
        elif property_name == "Volume":
            return GLib.Variant("d", MainPlayer.get().volume)
        else:
            return GLib.Variant("b", False)

    def GetAll(self, interface):
        ret = {}
        if interface == self.__MPRIS_IFACE:
            for property_name in ["CanQuit", "CanRaise", "Identity", "DesktopEntry"]:
                ret[property_name] = self.Get(interface, property_name)
        elif interface == self.__MPRIS_PLAYER_IFACE:
            for property_name in [
                "PlaybackStatus",
                "Metadata",
                "Volume",
                "CanGoNext",
                "CanGoPrevious",
                "CanPlay",
                "CanPause",
                "CanControl",
            ]:
                ret[property_name] = self.Get(interface, property_name)
        return ret

    def Set(self, interface, property_name, new_value):
        if property_name == "Volume":
            MainPlayer.get().volume = new_value

    def PropertiesChanged(
        self, interface_name, changed_properties, invalidated_properties
    ):
        self.__bus.emit_signal(
            None,
            self.__MPRIS_PATH,
            "org.freedesktop.DBus.Properties",
            "PropertiesChanged",
            GLib.Variant.new_tuple(
                GLib.Variant("s", interface_name),
                GLib.Variant("a{sv}", changed_properties),  # type: ignore
                GLib.Variant("as", invalidated_properties),
            ),
        )

    def Introspect(self):
        return self.__doc__

    def _get_status(self):
        playing = MainPlayer.get().playing
        if playing:
            return "Playing"
        else:
            return "Paused"

    def _on_preset_changed(self, _player, preset):
        self.__metadata["xesam:title"] = GLib.Variant("s", preset.name)
        changed_properties = {
            "Metadata": GLib.Variant("a{sv}", self.__metadata),
            "CanGoNext": GLib.Variant("b", MainPlayer.get().can_next),
            "CanGoPrevious": GLib.Variant("b", MainPlayer.get().can_prev),
        }
        self.PropertiesChanged(self.__MPRIS_PLAYER_IFACE, changed_properties, [])

    def _on_volume_changed(self, player, volume):
        self.PropertiesChanged(
            self.__MPRIS_PLAYER_IFACE,
            {
                "Volume": GLib.Variant("d", MainPlayer.get().volume),
            },
            [],
        )

    def _on_playing_changed(self, player, playing):
        properties = {"PlaybackStatus": GLib.Variant("s", self._get_status())}
        self.PropertiesChanged(self.__MPRIS_PLAYER_IFACE, properties, [])



# Copyright 2020-2022 Rafael Mardojai CM
# SPDX-License-Identifier: GPL-3.0-or-later

from gettext import gettext as _

from gi.repository import Gio, GLib, GObject, Gtk

from blanket.define import RES_PATH
from blanket.main_player import MainPlayer
from blanket.sound import Sound


@Gtk.Template(resource_path=f"{RES_PATH}/sound-context-menu.ui")
class SoundContextMenu(Gtk.PopoverMenu):
    __gtype_name__ = "SoundContextMenu"

    volume: Gtk.Scale = Gtk.Template.Child()  # type: ignore

    def __init__(self, sound: Sound):
        super().__init__()

        self.sound: Sound = sound

        if self.sound.custom:
            if index := MainPlayer.get().get_index(sound.name):
                # Set remove menu item
                custom_section = Gio.Menu()
                self.rename_item = Gio.MenuItem.new(_("Rename"), None)
                self.rename_item.set_action_and_target_value(
                    "app.rename-sound", GLib.Variant.new_uint32(index)
                )
                custom_section.append_item(self.rename_item)
                self.remove_item = Gio.MenuItem.new(_("Remove"), None)
                self.remove_item.set_action_and_target_value(
                    "app.remove-sound", GLib.Variant.new_uint32(index)
                )
                custom_section.append_item(self.remove_item)

                self.props.menu_model.append_section(None, custom_section)  # type: ignore

    @Gtk.Template.Callback()
    def on_opened(self, _popup):
        self.volume.set_value(self.sound.saved_volume)

        vol_adjustment = self.volume.get_adjustment()
        self.vol_bind = self.sound.bind_property(
            "saved_volume", vol_adjustment, "value", GObject.BindingFlags.BIDIRECTIONAL
        )

    def do_closed(self):
        self.vol_bind.unbind()



# Copyright 2021-2022 Rafael Mardojai CM
# SPDX-License-Identifier: GPL-3.0-or-later

from gettext import gettext as _

from gi.repository import Adw, Gio, Gtk

from blanket.define import RES_PATH
from blanket.preset import Preset
from blanket.settings import Settings


@Gtk.Template(resource_path=f"{RES_PATH}/preset-dialog.ui")
class PresetDialog(Adw.Dialog):
    __gtype_name__ = "PresetDialog"

    headerbar: Adw.HeaderBar = Gtk.Template.Child()  # type: ignore
    title_widget: Adw.WindowTitle = Gtk.Template.Child()  # type: ignore
    accept_btn: Gtk.Button = Gtk.Template.Child()  # type: ignore
    name_entry: Adw.EntryRow = Gtk.Template.Child()  # type: ignore

    def __init__(self, preset=None, **kwargs):
        super().__init__()

        self.preset = preset
        app = Gio.Application.get_default()
        self.window = app.get_active_window()  # type: ignore

        if self.preset is None:
            self.set_title(_("New Preset"))
            # Wire buttons
            self.accept_btn.connect("clicked", self._on_create_preset)
        else:
            self.set_title(_("Edit Preset"))
            self.title_widget.set_subtitle(self.preset.name)
            self.name_entry.set_text(self.preset.name)
            # Wire buttons
            self.accept_btn.connect("clicked", self._on_rename_preset)

        self.connect("realize", Gio.Application.get_default().unset_space_accel)  # type: ignore
        self.connect("closed", Gio.Application.get_default().set_space_accel)  # type: ignore

    @Gtk.Template.Callback()
    def _on_cancel_clicked(self, _button):
        self.close()

    @Gtk.Template.Callback()
    def _on_entry_changed(self, _entry):
        name = self.__get_name()

        if self.preset is not None and self.preset.name == name:
            self.accept_btn.set_sensitive(False)
            return

        if name:
            self.accept_btn.set_sensitive(True)
        else:
            self.accept_btn.set_sensitive(False)

    def _on_create_preset(self, _button):
        name = self.__get_name()
        if name:
            chooser = self.window.presets_chooser
            preset_id = Settings.get().add_preset(name)  # Save new preset
            # Add preset to model
            preset = Preset(preset_id)
            chooser.model.append(preset)

            # Select new preset
            chooser.selected = preset
        else:
            self.__invalid_name()
            return

        # Clear name entry
        self.name_entry.set_text("")

        self.close()

    def _on_rename_preset(self, _button):
        if self.preset:
            name = self.__get_name()
            if name:
                Settings.get().set_preset_name(self.preset.id, name)
            else:
                self.__invalid_name()
                return

        self.close()

    def __get_name(self):
        name = self.name_entry.get_text()
        name = name.strip()  # Strip name

        return name

    def __invalid_name(self):
        pass



# Copyright 2021-2022 Rafael Mardojai CM
# SPDX-License-Identifier: GPL-3.0-or-later

from gi.repository import Gio, GObject, Gtk

from blanket.define import RES_PATH
from blanket.settings import Settings
from blanket.widgets.preset_dialog import PresetDialog


@Gtk.Template(resource_path=f"{RES_PATH}/preset-row.ui")
class PresetRow(Gtk.ListBoxRow):
    __gtype_name__ = "PresetRow"

    custom: bool = GObject.Property(type=bool, default=False)  # type: ignore
    selected: bool = GObject.Property(type=bool, default=False)  # type: ignore

    name: Gtk.Label = Gtk.Template.Child()  # type: ignore
    rename_btn: Gtk.Button = Gtk.Template.Child()  # type: ignore
    delete_btn: Gtk.Button = Gtk.Template.Child()  # type: ignore

    def __init__(self, preset):
        super().__init__()

        self.preset = preset
        self.custom = self.preset.id != Settings.get().default_preset

        self.rename_btn.connect("clicked", self._on_show_rename)
        self.delete_btn.connect("clicked", self._on_delete_preset)

        preset.bind_property(
            "active", self, "selected", GObject.BindingFlags.SYNC_CREATE
        )

        preset.bind_property(
            "name", self.name, "label", GObject.BindingFlags.SYNC_CREATE
        )

        self.connect("notify::selected", self._on_selected_changed)

    def _on_selected_changed(self, _row, _pspec):
        if self.selected:
            self.add_css_class("selected")
        else:
            self.remove_css_class("selected")

    def _on_show_rename(self, _button):
        # Close popover
        popover: Gtk.Popover = self.get_native()  # type: ignore
        if popover is not None:
            popover.popdown()

        # Open edit dialog
        app = Gio.Application.get_default()
        if app:
            window = app.get_active_window()  # type: ignore
            if window:
                dialog = PresetDialog(self.preset)
                dialog.present(window)

    def _on_delete_preset(self, _button):
        app = Gio.Application.get_default()
        if app:
            window = app.get_active_window()  # type: ignore
            if window:
                active = self.preset.id == Settings.get().active_preset
                index = self.preset.remove()

                if index is not None:
                    chooser = window.presets_chooser
                    chooser.model.remove(index)

                    # Select default preset row
                    if active:
                        chooser.selected = chooser.model.get_item(0)



# Copyright 2020-2022 Rafael Mardojai CM
# SPDX-License-Identifier: GPL-3.0-or-later

from gi.repository import Gdk, GObject, Gtk

from blanket.define import RES_PATH
from blanket.sound import Sound
from blanket.widgets.sound_context_menu import SoundContextMenu


@Gtk.Template(resource_path=f"{RES_PATH}/sound-item.ui")
class SoundItem(Gtk.FlowBoxChild):
    __gtype_name__ = "SoundItem"

    playing: bool = GObject.Property(type=bool, default=False)  # type: ignore
    title: str = GObject.Property(type=str)  # type: ignore
    icon_name: str = GObject.Property(type=str)  # type: ignore
    sound: Sound = GObject.Property(type=Sound)  # type: ignore

    icon: Gtk.Image = Gtk.Template.Child()  # type: ignore
    label: Gtk.Label = Gtk.Template.Child()  # type: ignore
    volume: Gtk.Scale = Gtk.Template.Child()  # type: ignore

    def __init__(self):
        super().__init__()

        self._sound: Sound | None = None
        self._menu: None | SoundContextMenu = None

        self.connect("notify::playing", self._playing_changed)

        # Icon
        self.bind_property(
            "icon_name", self.icon, "icon_name", GObject.BindingFlags.SYNC_CREATE
        )
        # Label
        self.bind_property(
            "title", self.label, "label", GObject.BindingFlags.SYNC_CREATE
        )

        click = Gtk.GestureClick()
        click.set_button(3)  # Listen to secondary button (aka right-click)
        click.connect("pressed", self._on_secondary_click)
        self.add_controller(click)

    @GObject.Property(type=Sound)
    def sound(self) -> Sound | None:  # type: ignore
        return self._sound

    @sound.setter
    def sound(self, value: Sound):
        self._sound = value

        if self._sound is not None:
            vol_adjustment = self.volume.get_adjustment()

            vol_adjustment.props.value = self._sound.saved_volume
            self.volume.props.sensitive = self._sound.playing

            self._sound.bind_property(
                "saved_volume",
                vol_adjustment,
                "value",
                GObject.BindingFlags.BIDIRECTIONAL,
            )
            self._sound.bind_property(
                "playing", self.volume, "sensitive", GObject.BindingFlags.DEFAULT
            )

            self.volume.props.visible = True

    def _playing_changed(self, _object, _pspec):
        if not self.playing:
            self.icon.remove_css_class("accent")
        elif not self.icon.has_css_class("accent"):
            self.icon.add_css_class("accent")

    def _on_secondary_click(self, _ctrl, _n, x: int, y: int):
        self._context_popover(x, y)

    def _context_popover(self, x: int, y: int):
        if self.sound is None:
            return

        if self._menu is None:
            self._menu = SoundContextMenu(self.sound)

            rec = Gdk.Rectangle()
            rec.x = x
            rec.y = y

            self._menu.set_parent(self)
            self._menu.set_pointing_to(rec)

        self._menu.popup()



# Copyright 2021-2022 Rafael Mardojai CM
# SPDX-License-Identifier: GPL-3.0-or-later

from gi.repository import Gio, GObject, Gtk

from blanket.define import RES_PATH
from blanket.preset import Preset
from blanket.settings import Settings
from blanket.widgets.preset_row import PresetRow


@Gtk.Template(resource_path=f"{RES_PATH}/preset-chooser.ui")
class PresetChooser(Gtk.MenuButton):
    __gtype_name__ = "PresetChooser"

    selected: Preset = GObject.Property(type=Preset)  # type: ignore

    presets_list: Gtk.ListBox = Gtk.Template.Child()  # type: ignore

    def __init__(self):
        super().__init__()

        # Create GioListStore to store Presets
        self.model = Gio.ListStore.new(Preset)
        self.presets_list.bind_model(self.model, self._create_widget)
        self.connect("notify::selected", self._on_selected_changed)

        # Wire widgets
        self.presets_list.connect("row-activated", self._on_preset_activated)
        Settings.get().connect("preset-changed", self._on_saved_changed)

        self.load_presets()

    def load_presets(self):
        presets = Settings.get().get_presets_dict()
        for preset_id in presets:
            preset = Preset(preset_id)
            self.model.append(preset)

            if preset_id == Settings.get().active_preset:
                self.selected = preset

    def _on_preset_activated(self, _listbox, row: PresetRow):
        index = 0 if row is None else row.get_index()
        preset = self.model.get_item(index)
        if preset:
            self.selected = preset  # type: ignore

    def _on_saved_changed(self, _settings, _id):
        for preset in self.model:
            if isinstance(preset, Preset) and preset.active:
                self.selected = preset

    def _on_selected_changed(self, _chooser, _pspec):
        if self.selected is not None:
            if Settings.get().active_preset != self.selected.id:
                Settings.get().active_preset = self.selected.id

    def _create_widget(self, preset: Preset) -> Gtk.Widget:
        widget = PresetRow(preset)
        if preset.id == Settings.get().active_preset:
            widget.selected = True
        return widget



# Copyright 2022 Rafael Mardojai CM
# SPDX-License-Identifier: GPL-3.0-or-later

from blanket.widgets.playpause import PlayPauseButton  # noqa
from blanket.widgets.preset_chooser import PresetChooser  # noqa
from blanket.widgets.preset_dialog import PresetDialog  # noqa
from blanket.widgets.sound_item import SoundItem  # noqa
from blanket.widgets.volume_row import VolumeRow  # noqa



# Copyright 2020-2022 Rafael Mardojai CM
# SPDX-License-Identifier: GPL-3.0-or-later

from gi.repository import Gio, GObject, Gtk

from blanket.preset import Preset
from blanket.settings import Settings


class MainPlayer(GObject.GObject, Gio.ListModel):
    """
    Virtual app sounds player

    It also implements Gio.ListModel and stores the app sound list.
    """

    _instance = None
    _cookie = 0
    _sounds = []  # Sound list

    __gtype_name__ = "MainPlayer"
    __gsignals__ = {
        "preset-changed": (GObject.SIGNAL_RUN_FIRST, None, (GObject.Object,)),
        "reset-volumes": (GObject.SIGNAL_RUN_FIRST, None, ()),
    }

    playing: bool = GObject.Property(type=bool, default=True)  # type: ignore
    volume: float = GObject.Property(type=float, default=0)  # type: ignore

    @classmethod
    def get(cls):
        """Return an active instance of MainPlayer."""
        if cls._instance is None:
            cls._instance = MainPlayer()
        return cls._instance

    def __init__(self):
        super().__init__()
        self.connect("notify::playing", self._on_playing)
        Settings.get().connect("preset-changed", self._on_preset_changed)
        Settings.get().connect(
            "changed::inhibit-suspension", self._on_settings_inhibition
        )

        self.__add_item = GObject.GObject()  # Fake sound that adds new sounds
        self.__add_item.playing = False  # type: ignore

    def mute_vol_zero(self):
        for sound in self:
            if sound.saved_volume == 0:
                sound.playing = False

    def reset_volumes(self):
        self.emit("reset-volumes")

    def next_preset(self):
        if not self.can_next:
            return

        presets = Settings.get().presets
        index = presets.index(Settings.get().active_preset)
        Settings.get().active_preset = presets[index + 1]

    def prev_preset(self):
        if not self.can_prev:
            return

        presets = Settings.get().presets
        index = presets.index(Settings.get().active_preset)
        Settings.get().active_preset = presets[index - 1]

    @property
    def can_next(self) -> bool:
        presets = Settings.get().presets
        index = presets.index(Settings.get().active_preset)

        return index != len(presets) - 1 and len(presets) > 1

    @property
    def can_prev(self) -> bool:
        presets = Settings.get().presets
        index = presets.index(Settings.get().active_preset)

        return index > 0 and len(presets) > 1

    def _inhibit(self, inhibit: bool):
        """
        Set suspension inhibition
        """
        app = Gtk.Application.get_default()
        if inhibit:
            if self._cookie:
                return

            self._cookie = app.inhibit(  # type: ignore
                None, Gtk.ApplicationInhibitFlags.SUSPEND, "Playback in progress"
            )
        elif self._cookie != 0:
            app.uninhibit(self._cookie)  # type: ignore
            self._cookie = 0

    def _on_playing(self, _player, _param):
        """
        Toggle suspension inhibition when playing
        """
        if Settings.get().inhibit_suspension:
            self._inhibit(self.playing)

    def _on_settings_inhibition(self, _settings: Settings, _key: str):
        """
        Toggle suspension inhibition when settings changes
        """
        if Settings.get().inhibit_suspension:
            self._inhibit(self.playing)
        else:
            self._inhibit(False)

    def _on_preset_changed(self, _settings, preset_id):
        self.emit("preset-changed", Preset(preset_id))

    """
    ListModel methods

    This model keeps a fake last item that serves for the add sound item
    """

    def __iter__(self):
        return iter(self._sounds)

    def do_get_item(self, position: int) -> GObject.Object:
        if position == len(self._sounds):
            return self.__add_item  # Return plain GObject of Add sound item
        return self._sounds[position]

    def do_get_item_type(self):
        return GObject.Object

    def do_get_n_items(self) -> int:
        return len(self._sounds) + 1  # Add fake sound to total of items

    def append(self, sound: GObject.Object):
        self._sounds.append(sound)
        self.items_changed(len(self._sounds) - 1, 0, 1)

    def remove(self, position: int):
        del self._sounds[position]
        self.items_changed(position, 1, 0)

    def get_by_name(self, name: str) -> tuple[GObject.Object, int] | tuple[None, None]:
        for position, sound in enumerate(self._sounds):
            if sound.name == name:
                return (sound, position)

        return (None, None)

    def get_index(self, name: str) -> int | None:
        return self.get_by_name(name)[1]

    def get_by_index(self, index: int) -> GObject.Object:
        return self._sounds[index]



# Copyright 2020-2021 Rafael Mardojai CM
# SPDX-License-Identifier: GPL-3.0-or-later

import os
from gettext import gettext as _
from urllib.parse import unquote, urlparse

from gi.repository import Adw, Gio, GLib, GObject, Gtk

from blanket.define import RES_PATH, SOUNDS
from blanket.main_player import MainPlayer
from blanket.settings import Settings
from blanket.sound import Sound
from blanket.widgets import PlayPauseButton, PresetChooser, SoundItem, VolumeRow


@Gtk.Template(resource_path=f"{RES_PATH}/window.ui")
class BlanketWindow(Adw.ApplicationWindow):
    __gtype_name__ = "BlanketWindow"

    headerbar: Adw.HeaderBar = Gtk.Template.Child()  # type: ignore
    toast_overlay: Adw.ToastOverlay = Gtk.Template.Child()  # type: ignore
    grid: Gtk.FlowBox = Gtk.Template.Child()  # type: ignore
    playpause_btn: PlayPauseButton = Gtk.Template.Child()  # type: ignore
    volumes: Gtk.Popover = Gtk.Template.Child()  # type: ignore
    volume: Gtk.Scale = Gtk.Template.Child()  # type: ignore
    volume_box: Gtk.Box = Gtk.Template.Child()  # type: ignore
    volume_list: Gtk.ListBox = Gtk.Template.Child()  # type: ignore
    presets_chooser: PresetChooser = Gtk.Template.Child()  # type: ignore
    labels_group: Gtk.SizeGroup = Gtk.Template.Child()  # type: ignore
    power_toast: Adw.Toast = Gtk.Template.Child()  # type: ignore

    def __init__(self, **kwargs):
        super().__init__(**kwargs)

        # Set default window icon for window managers
        self.set_default_icon_name("com.rafaelmardojai.Blanket")

        self.setup_actions()
        # Setup widgets
        self.setup()
        # Setup volume
        self.setup_volume_menu()
        # Populate sounds
        self.populate_sounds()

    def setup(self):
        # Setup grid
        self.sounds_filter = Gtk.CustomFilter.new(
            match_func=self._hide_inactive_sounds_filter
        )
        self.sounds_model = Gtk.FilterListModel.new(
            model=MainPlayer.get(), filter=self.sounds_filter
        )
        self.grid.bind_model(self.sounds_model, self._create_sound_item)
        self.grid.connect("child-activated", self._on_sound_activate)

        # Wire playpause button
        MainPlayer.get().bind_property(
            "playing", self.playpause_btn, "playing", GObject.BindingFlags.SYNC_CREATE
        )

        # Show preset chooser
        self.presets_chooser.props.visible = len(Settings.get().presets) > 1
        Settings.get().connect("changed::presets", self._on_presets_changed)

    def setup_actions(self):
        # Close window action
        action = Gio.SimpleAction.new("close", None)
        action.connect("activate", lambda _action, _param: self.close())
        self.add_action(action)

        # Hide non active sounds
        action = Gio.SimpleAction.new_stateful(
            "hide-inactive",
            None,
            Settings.get()
            .get_preset_settings(Settings.get().active_preset)
            .get_value("hide-inactive"),
        )
        action.connect("change-state", self._on_hide_non_active)
        self.add_action(action)

    def setup_volume_menu(self):
        # Get volume scale adjustment
        vol_adjustment = self.volume.get_adjustment()
        # Bind volume scale value with main player volume
        vol_adjustment.bind_property(
            "value", MainPlayer.get(), "volume", GObject.BindingFlags.BIDIRECTIONAL
        )
        # Set volume scale value on first run
        self.volume.set_value(MainPlayer.get().volume)

        # Setup volume list
        self.volume_filter = Gtk.CustomFilter.new(match_func=lambda item: item.playing)
        model = Gtk.FilterListModel(model=MainPlayer.get(), filter=self.volume_filter)
        model.connect("items-changed", self._volume_model_changed)
        self.volume_box.props.visible = model.get_n_items() > 0
        self.volume_list.bind_model(model, self._create_vol_row)

        # Connect mainplayer preset-changed signal
        MainPlayer.get().connect_after("preset-changed", self._on_preset_changed)
        # Connect mainplayer reset-volumes signal
        MainPlayer.get().connect_after("reset-volumes", self._on_reset_volumes)

        self.volumes.connect("closed", self._volumes_popup_closed)

    def populate_sounds(self):
        """
        Populate default and saved sounds
        """

        # Self populate
        for g in SOUNDS:
            # Iterate sounds
            for s in g["sounds"]:
                # Create a new Sound
                sound = Sound(s["name"], title=s["title"])
                MainPlayer.get().append(sound)

        # Load saved custom audios
        for name, uri in Settings.get().custom_audios.items():
            # Check if file actually exists
            path = unquote(urlparse(uri).path)
            if os.path.exists(path):
                # Create a new Sound
                sound = Sound(name, uri=uri, custom=True)
                MainPlayer.get().append(sound)
            else:
                Settings.get().remove_custom_audio(name)

                alert = Adw.AlertDialog.new(
                    _("Sound Automatically Removed"),
                    _(
                        "The {name} sound is no longer accessible, so it has been removed"
                    ).format(name=f"<b><i>{name}</i></b>"),
                )
                alert.add_response("accept", _("Accept"))
                alert.props.body_use_markup = True
                alert.props.default_response = "accept"
                alert.props.close_response = "accept"
                alert.present(self)

    def open_audio(self):
        def on_response(dialog, result):
            try:
                gfiles = dialog.open_multiple_finish(result)
            except GLib.Error as e:
                if e.code != Gtk.DialogError.DISMISSED:
                    print(f"Error: {e.message}")
                return
            for gfile in gfiles:
                filename = gfile.get_path()
                if filename:
                    basename = os.path.basename(filename)
                    name = basename[: basename.rfind(".")]
                    uri = gfile.get_uri()

                    # Create a new Sound
                    sound = Sound(name, uri=uri, custom=True)
                    # Save to settings
                    GLib.idle_add(
                        Settings.get().add_custom_audio, sound.name, sound.uri
                    )
                    # Add Sound to SoundsGroup
                    MainPlayer.get().append(sound)

        filters = {
            "Supported audio files": [
                "audio/ogg",
                "audio/flac",
                "audio/x-wav",
                "audio/wav",
                "audio/mpeg",
                "audio/aac",
            ],
            "Ogg": ["audio/ogg"],
            "FLAC": ["audio/flac"],
            "WAV": ["audio/x-wav", "audio/wav"],
            "MP3": ["audio/mpeg"],
            "AAC": ["audio/aac"],
        }

        self.filechooser = Gtk.FileDialog.new()
        self.filechooser.set_title(_("Open audio"))
        self.filechooser.set_modal(True)

        filter_store = Gio.ListStore.new(Gtk.FileFilter)
        for f, mts in filters.items():
            audio_filter = Gtk.FileFilter()
            audio_filter.set_name(f)
            for mt in mts:
                audio_filter.add_mime_type(mt)
            filter_store.append(audio_filter)

        self.filechooser.set_filters(filter_store)

        self.filechooser.open_multiple(self, None, on_response)

    @Gtk.Template.Callback()
    def _on_narrow_window_apply(self, _breakpoint):
        if len(Settings.get().presets) > 1:
            self.headerbar.props.show_title = False

    @Gtk.Template.Callback()
    def _on_narrow_window_unapply(self, _breakpoint):
        self.headerbar.props.show_title = True

    def _hide_inactive_sounds_filter(self, item):
        return (
            not Settings.get().get_preset_hide_inactive(Settings.get().active_preset)
            or item.playing
        )

    def _on_hide_non_active(self, action, value: bool):
        action.set_state(value)
        Settings.get().set_preset_hide_inactive(Settings.get().active_preset, value)

        self.__update_filters()

    def _create_vol_row(self, sound):
        row = VolumeRow()

        row.volume = sound.saved_volume
        sound.bind_property(
            "saved_volume", row, "volume", GObject.BindingFlags.BIDIRECTIONAL
        )

        sound.bind_property("title", row, "title", GObject.BindingFlags.SYNC_CREATE)

        return row

    def _create_sound_item(self, sound):
        item = SoundItem()
        # Add label to size group
        self.labels_group.add_widget(item.label)

        if isinstance(sound, Sound):
            # Actual sound items
            item.sound = sound

            sound.bind_property(
                "playing", item, "playing", GObject.BindingFlags.SYNC_CREATE
            )
            sound.bind_property(
                "title", item, "title", GObject.BindingFlags.SYNC_CREATE
            )
            sound.bind_property(
                "icon_name", item, "icon_name", GObject.BindingFlags.SYNC_CREATE
            )
        else:
            # Add new sound item
            item.title = _("Add…")
            item.icon_name = "com.rafaelmardojai.Blanket-add-symbolic"

        return item

    def _on_sound_activate(self, _grid, item):
        # If item sound is None, then it's the Add sound item
        if item.sound is not None:
            # Toggle sound playing state
            item.sound.playing = not item.sound.playing
            # Update volumes list
            self.__update_filters()
        else:
            # Open add sound file chooser
            self.open_audio()

    def _on_presets_changed(self, _settings, _key):
        self.presets_chooser.props.visible = len(Settings.get().presets) > 1

    def _on_preset_changed(self, _player, preset):
        self.__update_filters()

    def _on_reset_volumes(self, _player):
        self.__update_filters()

    def _volume_model_changed(self, model, _pos, _del, _add):
        # Hide volumes list if empty
        self.volume_box.props.visible = model.get_n_items() > 0

    def _volumes_popup_closed(self, _popover):
        # Disable sounds with volume = 0
        MainPlayer.get().mute_vol_zero()
        self.__update_filters()

    def _on_add_sound_clicked(self, _group):
        self.open_audio()

    def __update_filters(self):
        self.sounds_filter.changed(Gtk.FilterChange.DIFFERENT)
        self.volume_filter.changed(Gtk.FilterChange.DIFFERENT)

    def show_power_toast(self):
        self.toast_overlay.add_toast(self.power_toast)

    def hide_power_toast(self):
        self.power_toast.dismiss()



# Copyright 2020-2021 Rafael Mardojai CM
# SPDX-License-Identifier: GPL-3.0-or-later

from gi.repository import GObject

from blanket.define import RES_PATH
from blanket.main_player import MainPlayer
from blanket.player import Player
from blanket.settings import Settings


class Sound(GObject.Object):
    """
    Describe a sound with it's properties
    """

    __gtype_name__ = "Sound"

    playing: bool = GObject.Property(type=bool, default=False)  # type: ignore
    title: str = GObject.Property(type=str)  # type: ignore
    icon_name: str = GObject.Property(type=str)  # type: ignore

    def __init__(
        self,
        name: str,
        uri: str | None = None,
        title: str | None = None,
        custom: bool | None = False,
    ):
        super().__init__()

        resource = f"resource:{RES_PATH}/sounds/{name}.ogg"
        icon = "com.rafaelmardojai.Blanket-{}-symbolic"

        # Internal player
        self._player = None

        # Sound properties
        self.name = name
        self.uri = uri if uri else resource
        self.title = title if title else name
        self.icon_name = icon.format("sound-wave" if custom else name)
        self.custom = custom

        # Playing state
        self.connect("notify::playing", self._playing_changed)
        if not self.saved_mute:
            self.playing = True

        # Connect mainplayer preset-changed signal
        MainPlayer.get().connect("preset-changed", self._on_preset_changed)
        # Connect mainplayer reset-volumes signal
        MainPlayer.get().connect("reset-volumes", self._on_reset_volumes)

    @property
    def player(self) -> Player:
        """Internal player"""
        if self._player is None:
            self._player = Player(self)
        return self._player

    @GObject.Property(type=float)
    def saved_volume(self) -> float:  # type: ignore
        return Settings.get().get_sound_volume(self.name)

    @saved_volume.setter
    def saved_volume(self, volume: float):
        volume = round(volume, 2)
        self.player.set_virtual_volume(volume)
        Settings.get().set_sound_volume(self.name, volume)

        if volume != 0 and not self.playing:
            self.playing = True

    @property
    def saved_mute(self) -> bool:
        return Settings.get().get_sound_mute(self.name)

    @saved_mute.setter
    def saved_mute(self, mute: bool):
        Settings.get().set_sound_mute(self.name, mute)

    def remove(self):
        """Remove sound if it is custom"""
        if self.custom:
            self.player.set_virtual_volume(0)
            Settings.get().remove_custom_audio(self.name)

    def _playing_changed(self, _object, _pspec):
        # Toggle player mute state
        if self.playing:
            if self.saved_volume > 0:
                self.player.set_virtual_volume(self.saved_volume)
            else:
                self.player.set_virtual_volume(0.5)
                self.saved_volume = 0.5
        else:
            self.player.set_virtual_volume(0)

        self.saved_mute = not self.playing  # Save playing state

    def _on_preset_changed(self, _player, _preset):
        self.notify("saved_volume")
        self.playing = not self.saved_mute

    def _on_reset_volumes(self, _player):
        self.saved_volume = 0.0
        self.playing = False



# Copyright 2022 Rafael Mardojai CM
# SPDX-License-Identifier: GPL-3.0-or-later

from gettext import gettext as _

RES_PATH = "/com/rafaelmardojai/Blanket"

SOUNDS = [
    {
        "name": _("Nature"),
        "sounds": [
            {"name": "rain", "title": _("Rain")},
            {"name": "storm", "title": _("Storm")},
            {"name": "wind", "title": _("Wind")},
            {"name": "waves", "title": _("Waves")},
            {"name": "stream", "title": _("Stream")},
            {"name": "birds", "title": _("Birds")},
            {"name": "summer-night", "title": _("Summer Night")},
        ],
    },
    {
        "name": _("Travel"),
        "sounds": [
            {"name": "train", "title": _("Train")},
            {"name": "boat", "title": _("Boat")},
            {"name": "city", "title": _("City")},
        ],
    },
    {
        "name": _("Interiors"),
        "sounds": [
            {"name": "coffee-shop", "title": _("Coffee Shop")},
            {"name": "fireplace", "title": _("Fireplace")},
        ],
    },
    {
        "name": _("Noise"),
        "sounds": [
            {"name": "pink-noise", "title": _("Pink Noise")},
            {"name": "white-noise", "title": _("White Noise")},
        ],
    },
]

AUTHORS = ["Rafael Mardojai CM"]

ARTISTS = {"Jakub Steiner": [_("Sounds icons")], "Rafael Mardojai CM": [_("App icon")]}

SOUND_ARTISTS = {
    "alex36917": [_("Rain")],
    "Digifish music": [_("Storm")],
    "ezwa": [_("Fireplace")],
    "Falcet": [_("Boat")],
    "gezortenplotz": [_("City")],
    "gluckose": [_("Stream")],
    "Jorge Stolfi": [_("White Noise")],
    "kvgarlic": [_("Birds")],
    "Lisa Redfern": [_("Summer Night")],
    "Luftrum": [_("Waves")],
    "stephan": [_("Coffee Shop")],
    "Omegatron": [_("Pink Noise")],
    "felix.blume": [_("Wind")],
    "SDLx": [_("Train")],
}

SOUND_EDITORS = {
    "Porrumentzio": [
        _("Birds"),
        _("Rain"),
        _("Storm"),
        _("Waves"),
        _("Boat"),
        _("City"),
        _("Wind"),
    ]
}



# Copyright 2020-2022 Rafael Mardojai CM
# SPDX-License-Identifier: GPL-3.0-or-later

from gi.repository import GObject, Gtk

from blanket.define import RES_PATH


@Gtk.Template(resource_path=f"{RES_PATH}/volume-row.ui")
class VolumeRow(Gtk.ListBoxRow):
    __gtype_name__ = "VolumeRow"

    title: str = GObject.Property(type=str)  # type: ignore
    volume: float = GObject.Property(type=float)  # type: ignore

    def __init__(self):
        super().__init__()
